/* **********************************************************************
 * Author      : Severin Willisch
 * email       : d9840087@mail.connect.usq.edu.au
 * Student No. : 0039840087
 * Purpose     : Project for Semester 1 & 2, 2009
 * Purpose     : 09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class monitors a specific input to tigger an ...
 * Purpose     :  alarm. If the level is low everything is normal, ...
 * Purpose     :  i.e. no alarm has been activated. The input is ...
 * Purpose     :  checked once every second. If an alarm is triggered ...
 * Purpose     :  a broadcast message is sent out. After the message ...
 * Purpose     :  has been sent, the thread waits until the the D2 pin ...
 * Purpose     :  has been set to low. This prevents the SPOT from ...
 * Purpose     :  sending out message that alarm has been triggered. ...
 * Purpose     :  After input has been set to low the thread continues ...
 * Purpose     :  to monitor the D2 input.
 * Inputs      :  Location of mote,
                  port used by host,
                  data type,
                  sample time
 * Outputs     :  Alarm trigger sent via radio
 * **********************************************************************
 */

package org.sunspotworld;

class AlarmTriggerThread implements Runnable {

   private String location;
   private Thread t;
   private int hostPortNumber;
   private byte dataIdentifier;
   private long sampleTime;
   private long timeToBeSent;


   AlarmTriggerThread(String threadname, int portNo, byte dataType,
           long timeBetweenSamples) {

      location = threadname;
      t = new Thread(this, location);
      hostPortNumber = portNo;
      dataIdentifier = dataType;
      sampleTime = timeBetweenSamples;
      System.out.println("New thread: " + t);
      t.start();
   }

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setLocThread(String setLoc){
      location = setLoc;
   }

   void setThread(Thread setThread){
      t = setThread;
   }

   void setPortThread(int setPort){
      hostPortNumber = setPort;
   }

   void setDataIdType(byte setDataType){
      dataIdentifier = setDataType;
   }

   void setTimeBetweenSamples(long setSampleInterval){
      sampleTime = setSampleInterval;
   }

   void setTimeForSending(long setSendTime){
      timeToBeSent = setSendTime;
   }
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   String getLocThread(){
      return location;
   }

   Thread getThread(){
      return t;
   }

   int getPortThread(){
      return hostPortNumber;
   }

   byte getDataIdType(){
      return dataIdentifier;
   }

   long getTimeBetweenSamples(){
      return sampleTime;
   }

   long getTimeForSending(){
      return timeToBeSent;
   }

   // ===================================================================


   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // entry point for alarm trigger thread
   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

   public void run() {

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start alarm thread: " + location);
      // ================================================================

      // ================================================================
      // start a new alarm trigger mote
      // ----------------------------------------------------------------
      CollectorAlarmMote moteAlarm = new CollectorAlarmMote();
      // ================================================================

      // ================================================================
      // Initial set up for alarm trigger
      // ----------------------------------------------------------------
      moteAlarm.initAlarm();
      // ================================================================

      // ================================================================
      // Initial set up for communication
      // ----------------------------------------------------------------
      moteAlarm.initBroadcastCom();
      moteAlarm.setPortNumber(hostPortNumber);
      moteAlarm.setLocation(location);
      moteAlarm.setWriteDataByte(dataIdentifier);
      // ================================================================

      // ================================================================
      // get and push IEEE Address to terminal
      // ----------------------------------------------------------------
      moteAlarm.getPushAddress();
      // ================================================================

      // ================================================================
      // Open up a broadcast connection
      // ----------------------------------------------------------------
      moteAlarm.tryToOpenBroadcastConnection();
      // ================================================================
      
      while (true) {

         // =============================================================
         // Check if input is low. Low means no alarm triggered
         // -------------------------------------------------------------
         while (moteAlarm.pinStatus()) {
            // ==========================================================
            // if alarm is not triggerd go to sleep for specified time
            // ----------------------------------------------------------
            moteAlarm.sleepForSomeTime(sampleTime,timeToBeSent);
            // ==========================================================

            // ==========================================================
            // Get sample time
            // ----------------------------------------------------------
            timeToBeSent = moteAlarm.getCurrentTime();
            // ==========================================================

            System.out.println("timeToBeSent: " + timeToBeSent);
            System.out.println("sampleTime  : " + sampleTime);

         } // end of while (pin.isLow()) loop
         // =============================================================

         // =============================================================
         // The application only gets to here because pin is no longer...
         // ... low. Therefore, send message to notify that an alarm ...
         // ... has been triggered.
         // =============================================================
         
         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Send data");
         // =============================================================

         // =============================================================
         // Update alarm values to be sent
         // -------------------------------------------------------------
         moteAlarm.setWriteDataLong(timeToBeSent);
         moteAlarm.setWriteDataInt(1);             // 1 = alarm triggered
         // =============================================================

         // =============================================================
         // Package time & alarm trigger into a radio datagram & send
         // -------------------------------------------------------------
         moteAlarm.packageAndSendRadiogram();
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Data sent");
         // =============================================================

         // =============================================================
         // Stop the thread until the input has been set to low. Only ...
         // send "alarm triggered" once. Wait for pin D2 to change ...
         // state. Once the pin has changed renable pin change ...
         // interrupt (listen for new alarm)
         // -------------------------------------------------------------
         moteAlarm.getPinStatus().waitForChange();
         moteAlarm.setEnablePinChangeInterruptsToTrue();
         // =============================================================
         
         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("After waitForChange() statement");
         // =============================================================

         // =============================================================
         // The input has changed back to normal. Therefore send info ...
         // to notify of return to "No alarm". First verify that the ...
         // the input is back to normal postion, then send datagram.
         // =============================================================
         if (moteAlarm.pinStatus()) {
            try {
               // =======================================================
               // For trouble shooting only
               // -------------------------------------------------------
               System.out.println("Send data");
               // =======================================================

               // =======================================================
               // Get the current time
               // -------------------------------------------------------
               timeToBeSent = moteAlarm.getCurrentTime();
               // =======================================================

               // =======================================================
               // Update alarm values to be sent
               // -------------------------------------------------------
               moteAlarm.setWriteDataLong(timeToBeSent);
               moteAlarm.setWriteDataInt(2);   // 2 = alarm not triggered
               // =======================================================

               // =======================================================
               // Package time & alarm trigger into radio datagram & send
               // -------------------------------------------------------
               moteAlarm.packageAndSendRadiogram();
               // =======================================================

               // =======================================================
               // For trouble shooting only
               // -------------------------------------------------------
               System.out.println("Data sent");
               // =======================================================

            } catch (Exception ex) {
               System.out.println("Problem sending datagram" + ex);
            }
         } // end of if(pin.isLow())
      } // end of loop always

   } // end of run()
} // end of class
