/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class runs if the MANIFEST.MF file has battery ...
 * Purpose     :  measurement set to TRUE.
 * Inputs      :  Location of mote,
                  port used by host,
                  data type,
                  sample time
 * Outputs     :  Environmental data collected
  **********************************************************************
 */
package org.sunspotworld;

import com.sun.spot.peripheral.Battery.*;

class BatLevelThread implements Runnable {

   private String location;                            // name of thread
   private Thread t;
   private int hostPortNumber;
   private byte dataIdentifier;
   private long sampleTime;
   private long timeToBeSent;
   private int battToBeSent;

   BatLevelThread(String threadname, int portNo, byte dataType,
           long timeBetweenSamples) {
      location = threadname;
      t = new Thread(this, location);
      hostPortNumber = portNo;
      dataIdentifier = dataType;
      sampleTime = timeBetweenSamples;
      System.out.println("New thread: " + t);
      t.start();
   }
   
   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setLocThread(String setLoc){
      location = setLoc;
   }

   void setThread(Thread setThread){
      t = setThread;
   }

   void setPortThread(int setPort){
      hostPortNumber = setPort;
   }

   void setDataIdType(byte setDataType){
      dataIdentifier = setDataType;
   }

   void setTimeBetweenSamples(long setSampleInterval){
      sampleTime = setSampleInterval;
   }

   void setTimeForSending(long setSendTime){
      timeToBeSent = setSendTime;
   }

   void setBattForSending(int setSendBatt){
      battToBeSent = setSendBatt;
   }
   // ===================================================================


   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   String getLocThread(){
      return location;
   }

   Thread getThread(){
      return t;
   }

   int getPortThread(){
      return hostPortNumber;
   }

   byte getDataIdType(){
      return dataIdentifier;
   }

   long getTimeBetweenSamples(){
      return sampleTime;
   }

   long getTimeForSending(){
      return timeToBeSent;
   }

   int getBattForSending(){
      return battToBeSent;
   }
   // ===================================================================


   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // entry point for battery level thread
   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

   public void run() {
      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start battery thread: " + location);
      // ================================================================

      // ================================================================
      // Start a new battery level collector mote
      // ----------------------------------------------------------------
      CollectorBattMote moteBatt = new CollectorBattMote();
      // ================================================================

      // ================================================================
      // Intial set up for measurement of battery level
      // ----------------------------------------------------------------
      moteBatt.initBattMeasurement();
      // ================================================================

      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      moteBatt.initBroadcastCom();
      moteBatt.setPortNumber(hostPortNumber);
      moteBatt.setLocation(location);
      moteBatt.setWriteDataByte(dataIdentifier);
      // ================================================================

      // ================================================================
      // get and push IEEE Address to terminal
      // ----------------------------------------------------------------
      moteBatt.getPushAddress();
      // ================================================================

      // ================================================================
      // Open up a broadcast connection
      // ----------------------------------------------------------------
      moteBatt.tryToOpenBroadcastConnection();
      // ================================================================

      // ================================================================
      // Get the current battery reading and send in datagram
      // ----------------------------------------------------------------
      while (true){
         // =============================================================
         // Get the current time. I.e. time when sample is taken
         // -------------------------------------------------------------
         timeToBeSent = moteBatt.getCurrentTime();
         // =============================================================

         // =============================================================
         // Indicate sample event (start, i.e LED on)
         // -------------------------------------------------------------
         moteBatt.sampleEventIndication(true, 3);
         // =============================================================

         // =============================================================
         // Get the current battery reading to send in datagram
         // -------------------------------------------------------------
         battToBeSent = moteBatt.takeBattSample();
         // =============================================================

         // =============================================================
         // Indicate sample event (end, i.e LED off)
         // -------------------------------------------------------------
         moteBatt.sampleEventIndication(false, 3);
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Has battery capacity changed? " +
                 moteBatt.hasBattChanged());
         // =============================================================

         if(moteBatt.hasBattChanged()){
            // ==========================================================
            // For trouble shooting only
            // ----------------------------------------------------------
            System.out.println("Try to send sample.");
            // ==========================================================

            // ==========================================================
            // Update time and battery value to be sent
            // ----------------------------------------------------------
            moteBatt.setWriteDataLong(timeToBeSent);
            moteBatt.setWriteDataInt(battToBeSent);
            // ==========================================================

            // ==========================================================
            // Send current sample
            // ----------------------------------------------------------
            moteBatt.packageAndSendRadiogram();
            // ==========================================================

         } // end if block

         // =============================================================
         // move current battery sample to previous sample variable
         // -------------------------------------------------------------
         moteBatt.moveCurrentBattToPreviousBatt();
         // =============================================================

         // =============================================================
         // Go to sleep to conserve battery
         // -------------------------------------------------------------
         moteBatt.sleepForSomeTime(sampleTime, timeToBeSent);
         // =============================================================

      } // end of while loop
   } // end of run()
} // end of class
