/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class runs if the MANIFEST.MF file has carbon ...
 * Purpose     :  mono measurement set to TRUE.
 * Inputs      :  Location of mote,
 *                port used by host,
 *                data type,
 *                sample time
 * Outputs     :  Enviromental data collected
 * **********************************************************************
 */
package org.sunspotworld;

class CarMonThread implements Runnable {

   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private String location;
   private Thread t;
   private int hostPortNumber;
   private byte dataIdentifier;
   private long sampleTime;
   private long timeToBeSent;
   private double carbonMonoToBeSent;
   private int currentCarbonMonoSample;
   // ===================================================================

   CarMonThread(String threadname, int portNo, byte dataType,
           long timeBetweenSamples) {
      location = threadname;
      t = new Thread(this, location);
      hostPortNumber = portNo;
      dataIdentifier = dataType;
      sampleTime = timeBetweenSamples;
      System.out.println("New thread: " + t);
      t.start();
   }

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setLocThread(String setLoc) {
      location = setLoc;
   }

   void setThread(Thread setThread) {
      t = setThread;
   }

   void setPortThread(int setPort) {
      hostPortNumber = setPort;
   }

   void setDataIdType(byte setDataType) {
      dataIdentifier = setDataType;
   }

   void setTimeBetweenSamples(long setSampleInterval) {
      sampleTime = setSampleInterval;
   }

   void setTimeForSending(long setSendTime) {
      timeToBeSent = setSendTime;
   }

   void setTempForSending(double setSendTemp) {
      carbonMonoToBeSent = setSendTemp;
   }
   // ===================================================================


   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   String getLocThread() {
      return location;
   }

   Thread getThread() {
      return t;
   }

   int getPortThread() {
      return hostPortNumber;
   }

   byte getDataIdType() {
      return dataIdentifier;
   }

   long getTimeBetweenSamples() {
      return sampleTime;
   }

   long getTimeForSending() {
      return timeToBeSent;
   }

   double getTempForSending() {
      return carbonMonoToBeSent;
   }
   // ===================================================================


   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // entry point for carbon mono thread
   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   public void run() {

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start carbon mono thread: " + location);
      // ================================================================

      // ================================================================
      // start a new carbon monoxide collector mote
      // ----------------------------------------------------------------
      CollectorCarbonMonoMote moteCarMon = new CollectorCarbonMonoMote();
      // ================================================================

      // ================================================================
      // Intial set up for measurement of carbon monoxide
      // ----------------------------------------------------------------
      moteCarMon.initCarbonMonoMeasurement();
      // ================================================================

      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      moteCarMon.initBroadcastCom();
      moteCarMon.setPortNumber(hostPortNumber);
      moteCarMon.setLocation(location);
      moteCarMon.setWriteDataByte(dataIdentifier);
      // ================================================================

      // ================================================================
      // get and push IEEE Address to terminal
      // ----------------------------------------------------------------
      moteCarMon.getPushAddress();
      // ================================================================

      // ================================================================
      // Open up a broadcast connection
      // ----------------------------------------------------------------
      moteCarMon.tryToOpenBroadcastConnection();
      // ================================================================

      // ================================================================
      // read the current value on the A0 input
      // ----------------------------------------------------------------
      while (true) {
         // =============================================================
         // Get the current time. I.e. time when sample is taken
         // -------------------------------------------------------------
         timeToBeSent = moteCarMon.getCurrentTime();
         // =============================================================

         // =============================================================
         // Indicate sample event (start, i.e LED on)
         // -------------------------------------------------------------
         moteCarMon.sampleEventIndication(true, 5);
         // =============================================================

         // =============================================================
         // Get the current carbon mono reading to send in datagram
         // -------------------------------------------------------------
         currentCarbonMonoSample = moteCarMon.takeCarbonMonoSample();
         // =============================================================

         // =============================================================
         // Indicate sample event (end, i.e LED off)
         // -------------------------------------------------------------
         moteCarMon.sampleEventIndication(false, 5);
         // =============================================================

         // =============================================================
         // Convert Sample to volt
         // -------------------------------------------------------------
         carbonMonoToBeSent = moteCarMon.
                 convertCarbonMonoSampleToVolt(currentCarbonMonoSample);
         // =============================================================

         // =============================================================
         // Reformat sample to be sent to ****.** format
         // -------------------------------------------------------------
         carbonMonoToBeSent = moteCarMon.reformatDataToBeSent(
                 carbonMonoToBeSent);
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Has carbon mono level changed? " +
                 moteCarMon.hasCarbonMonoLevelChanged());
         // =============================================================

         if (moteCarMon.hasCarbonMonoLevelChanged()) {
            // ==========================================================
            // For trouble shooting only
            // ----------------------------------------------------------
            System.out.println("Try to send sample.");
            // ==========================================================

            // ==========================================================
            // Update time and carbon mono value to be sent
            // ----------------------------------------------------------
            moteCarMon.setWriteDataLong(timeToBeSent);
            moteCarMon.setWriteDataDouble(carbonMonoToBeSent);
            // ==========================================================

            // ==========================================================
            // Send current sample
            // ----------------------------------------------------------
            moteCarMon.packageAndSendRadiogram();
            // ==========================================================

         } // end if block
         
         // =============================================================
         // move current carbon level to previous carbon level variable
         // -------------------------------------------------------------
         moteCarMon.moveCurrentCarbonMonoToPreviousCarbonMono();
         // =============================================================

         // =============================================================
         // Go to sleep to conserve battery
         // -------------------------------------------------------------
         moteCarMon.sleepForSomeTime(sampleTime, timeToBeSent);
         // =============================================================

      } //end of while loop
   } // end of run()
} // end of class
