/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Description :  Main class for the collector SPOTs.
 * Inputs      :  MANIFEST.MF file properties
 * Outputs     :  None
 * Purpose     :  Reads properties from MANIFEST.MF file & configures ...
 * Purpose     :  ... collector SPOT based on the MANIFEST.MF file.
 * Purpose     :  After the appropriate functions (threads) have been ...
 * Purpose     :  ... started the class waits for all threads to finish.
 * Purpose     :  When all treads are finished, class finishes.
 * **********************************************************************
 */

package org.sunspotworld;

import com.sun.spot.util.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;


public class CollectMainSPOT extends MIDlet {
   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------

   // -------------------------------------------------------------------
   // Define constants
   // -------------------------------------------------------------------
   private static final int HOST_PORT = 67;    // Broadcast port
   private static final long SAMPLE_TIME_DEFAULT = 1000;
   // -------------------------------------------------------------------

   // -------------------------------------------------------------------
   // Manifest file configuration information will be saved to the ...
   // ... following variables
   // -------------------------------------------------------------------
   private String MANI_LOCATION = null;            // location of SPOT
   private String MANI_IDENT_TEMP = null;          // IDs temp data
   private String MANI_TEMPERATURE = null;         // temp measure
   private String MANI_IDENT_LIGHT = null;         // IDs light data
   private String MANI_LIGHT_LEVEL = null;         // light measure
   private String MANI_IDENT_BAT = null;           // IDs bat data
   private String MANI_BATTERY = null;             // battery measure
   private String MANI_IDENT_ALARM = null;         // IDs alarm data
   private String MANI_ALARM = null;               // alarm required
   private String MANI_IDENT_CARBON_MONO = null;   // Ids carbmon data
   private String MANI_CARBON_MONO = null;         // carbon monoxide
   // -------------------------------------------------------------------

   // ===================================================================
   // The data from the manifest file is of type String. Therefore ...
   // ... to make further processing easier they will be converted to ...
   // ... variables of type byte or int.
   // -------------------------------------------------------------------
   private byte identByteTemp = 0;
   private byte identByteLight = 0;
   private byte identByteBat = 0;
   private byte identByteAlarm = 0;
   private byte identByteCarbonMono = 0;
   private int tempInt = 0;
   private int lightInt = 0;
   private int batInt = 0;
   private int alarmInt = 0;
   private int carbonInt = 0;
   // -------------------------------------------------------------------

   // -------------------------------------------------------------------
   // Create boolean variables to set functions a SPOT needs. These ...
   // ... variables will be set based on the entries in the manifest.
   // -------------------------------------------------------------------
   private boolean tempFlag = false;      // true if MANI_TEMPERATURE = 1
   private boolean lightFlag = false;     // true if MANI_LIGHT_LEVEL = 1
   private boolean batFlag = false;       // true if MANI_BATTERY = 1
   private boolean alarmFlag = false;     // true if MANI_ALARM = 1
   private boolean carbonMonoFlag = false;// true if MANI_CARBON_MONO = 1
   // ===================================================================

   protected void startApp() {
      // ================================================================
      // monitor the USB (if connected) and recognize commands from host
      // ----------------------------------------------------------------
      new BootloaderListener().start();
      // ================================================================

      // ================================================================
      // Names of the different functions a SPOT can have. Function...
      // ... will only start if the manifest file instructs to run.
      // Each function will start a new thread.
      // ----------------------------------------------------------------
      BatLevelThread measureBat = null;         // place holder for now
      TempThread measureTemp = null;            // place holder for now
      LightThread measureLight = null;          // place holder for now
      AlarmTriggerThread detectAlarm = null;   // place holder for now
      CarMonThread detectCarbonMono = null;     // place holder for now
      // ================================================================

      // ================================================================
      // Get configuration data from Manifest file. After data has ...
      // ... been read from manifest it will be used to configure the ...
      // ... SPOT.
      // ----------------------------------------------------------------
      try {
         MANI_LOCATION = this.getAppProperty("Location");
         MANI_IDENT_TEMP = this.getAppProperty("IdentTemp");
         MANI_TEMPERATURE = this.getAppProperty("Temperature");
         MANI_IDENT_LIGHT = this.getAppProperty("IdentLight");
         MANI_LIGHT_LEVEL = this.getAppProperty("LightLevel");
         MANI_IDENT_BAT = this.getAppProperty("IdentBat");
         MANI_BATTERY = this.getAppProperty("Battery");
         MANI_IDENT_ALARM = this.getAppProperty("IdentAlarm");
         MANI_ALARM = this.getAppProperty("Alarm");
         MANI_IDENT_CARBON_MONO = this.getAppProperty("IdentCarbonMono");
         MANI_CARBON_MONO = this.getAppProperty("CarbonMono");

         // -------------------------------------------------------------
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Configuration data OK");
         // -------------------------------------------------------------

      } catch (NullPointerException e) {
         System.out.println("Problem reading MANIFEST.MF " + e);
      } // end try / catch
      // ================================================================


      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("The location is: " + MANI_LOCATION);
      // ================================================================

      // ================================================================
      // convert Strings to appropriate data type
      // ----------------------------------------------------------------
      try {
         identByteTemp = Byte.parseByte(MANI_IDENT_TEMP);
         tempInt = Integer.parseInt(MANI_TEMPERATURE);
         identByteLight = Byte.parseByte(MANI_IDENT_LIGHT);
         lightInt = Integer.parseInt(MANI_LIGHT_LEVEL);
         identByteBat = Byte.parseByte(MANI_IDENT_BAT);
         batInt = Integer.parseInt(MANI_BATTERY);
         identByteAlarm = Byte.parseByte(MANI_IDENT_ALARM);
         alarmInt = Integer.parseInt(MANI_ALARM);
         identByteCarbonMono = Byte.parseByte(MANI_IDENT_CARBON_MONO);
         carbonInt = Integer.parseInt(MANI_CARBON_MONO);
      } catch (NumberFormatException numberFormatException) {
         System.out.println("Problem converting data: "
                 + numberFormatException
         );
      } // end try / catch
      // ================================================================

      // ================================================================
      // Set flags for functions, if 1 --> true --> function required
      // ----------------------------------------------------------------
      if (tempInt == 1) {tempFlag = true;}         // temperature
      if (lightInt == 1) {lightFlag = true;}       // light
      if (batInt == 1) {batFlag = true;}           // battery capacity
      if (alarmInt == 1) {alarmFlag = true;}       // alarm
      if (carbonInt == 1) {carbonMonoFlag = true;} //carbon monoxide
      // ================================================================

      // ================================================================
      // start new threads on a SunSPOT depending Manifest file
      // ----------------------------------------------------------------
      if (tempFlag) {
         long SampleTimeTempStatus =
                 Long.parseLong(
                 this.getAppProperty("SampleTimeTemp")
                 );
         measureTemp = new TempThread(
              MANI_LOCATION, HOST_PORT, identByteTemp,
              SampleTimeTempStatus * SAMPLE_TIME_DEFAULT
         );
      } // end if (tempFlag)

      if (lightFlag) {
         long SampleTimeLightStatus =
                 Long.parseLong(
                 this.getAppProperty("SampleTimeLight")
                 );
         measureLight = new LightThread(
              MANI_LOCATION, HOST_PORT, identByteLight,
              SampleTimeLightStatus * SAMPLE_TIME_DEFAULT
         );
      } // end if (lightFlag)

      if (batFlag) {
         long SampleTimeBattStatus =
                 Long.parseLong(
                 this.getAppProperty("SampleTimeBattery")
                 );
         measureBat = new BatLevelThread(
              MANI_LOCATION, HOST_PORT, identByteBat,
              SampleTimeBattStatus * SAMPLE_TIME_DEFAULT
         );
      } // end if (batFlag)

      if (alarmFlag) {
         long SampleTimeAlarmStatus =
                 Long.parseLong(
                 this.getAppProperty("SampleTimeAlarm")
                 );
         detectAlarm = new AlarmTriggerThread(
              MANI_LOCATION, HOST_PORT, identByteAlarm,
              SampleTimeAlarmStatus * SAMPLE_TIME_DEFAULT
         );
      } // end if (alarmFlag)

      if (carbonMonoFlag) {
         long SampleTimeCarbonMonoStatus =
                 Long.parseLong(
                 this.getAppProperty("SampleTimeCarbonMono")
                 );
         detectCarbonMono = new CarMonThread(
              MANI_LOCATION, HOST_PORT, identByteCarbonMono,
              SampleTimeCarbonMonoStatus * SAMPLE_TIME_DEFAULT
         );
      } // end if (carbonMonoFlag)
      // ================================================================

      // ================================================================
      // Wait for functions (threads) to end
      // ----------------------------------------------------------------
      try {
         System.out.println("Wait for threads to finish.");
         if (measureTemp != null) {measureTemp.getThread().join();}

         if (measureLight != null) {measureLight.getThread().join();}

         if (measureBat != null) {measureBat.getThread().join();}

         if (detectAlarm != null) {detectAlarm.getThread().join();}

         if (detectCarbonMono != null) {detectCarbonMono.getThread().
                 join();}

      } catch (InterruptedException e) {
         System.out.println("Main thread interrupted." + e);
      } // end try / catch
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Main thread exiting.");
      // ================================================================

      // ================================================================
      // cause the midlet to exit
      // ----------------------------------------------------------------
      notifyDestroyed();
      // ================================================================
   } // end startApp()

   protected void pauseApp() {
      // This is not currently called by the Squawk VM
   } // end pauseApp()

   /**
    * Called if the MIDlet is terminated by the system.
    */
   protected void destroyApp(boolean arg0) throws
           MIDletStateChangeException {
      
   } // end destroyApp()
} // end class
