/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class includes all the common functions of a ...
 * Purpose     :  collector mote. Usually this class will be extended ...
 * Purpose     :  to add the specific capability for the collection ...
 * Purpose     :  of the environmental data.
 * **********************************************************************
 */

package org.sunspotworld;

import com.sun.spot.io.j2me.radiogram.RadiogramConnection;
import com.sun.spot.sensorboard.EDemoBoard;
import com.sun.spot.sensorboard.peripheral.ITriColorLED;
import com.sun.spot.util.Utils;
import com.sun.squawk.util.MathUtils;
import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.Datagram;


class CollectorMote {

   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private RadiogramConnection rCon;
   private Datagram dg;
   private String location;
   private int portNumber;

   private long writeDataLong = 0;
   private byte writeDataByte = 0;
   private int writeDataInt = 0;
   private double writeDataDouble = 0;
   private boolean writeDataBool = false;

   private ITriColorLED[] sampleEvent = EDemoBoard.
           getInstance().getLEDs();
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setLocation(String setLoc){
      location = setLoc;
   }

   void setPortNumber(int setPort){
      portNumber = setPort;
   }

   void setWriteDataLong(long setDataLong){
      writeDataLong = setDataLong;
   }

   void setWriteDataByte(byte setDataByte){
      writeDataByte = setDataByte;
   }

   void setWriteDataInt(int setDataInt){
      writeDataInt = setDataInt;
   }

   void setWriteDataDouble(double setDataDouble){
      writeDataDouble = setDataDouble;
   }

   void setWriteDataBoolean(boolean setDataBoolean){
      writeDataBool = setDataBoolean;
   }

   // ===================================================================

   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   String getLocation(){
      return location;
   }

   int getPortNumber(){
      return portNumber;
   }

   long getWriteDataLong(){
      return writeDataLong;
   }

   byte getWriteDataByte(){
      return writeDataByte;
   }

   int getWriteDataInt(){
      return writeDataInt;
   }

   double getWriteDataDouble(){
      return writeDataDouble;
   }

   boolean getWriteDataBoolean(){
      return writeDataBool;
   }

   // ===================================================================


   // ===================================================================
   // All other methods
   // ===================================================================
   void initBroadcastCom (){
      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      rCon = null;
      dg = null;
      // ================================================================

   } // end initBroadcastCom

   void getPushAddress() {
      // ================================================================
      // get and push IEEE Address to terminal. Troubleshoot only
      // ----------------------------------------------------------------
      String ourAddress = System.getProperty("IEEE_ADDRESS");
      System.out.println("Starting app. on " + ourAddress + " ...");
      // ================================================================

   } // end getPushAddress()

   void tryToOpenBroadcastConnection(){
      try {
         // =============================================================
         // Open up a broadcast connection to the host port where the ...
         // ... the host will be listening.
         // -------------------------------------------------------------
         rCon = (RadiogramConnection) Connector.open(
              "radiogram://broadcast:" + portNumber);
         dg = rCon.newDatagram(rCon.getMaximumLength());
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Broadcast is open.");
         // =============================================================
         
      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end tryToOpenBroadcastConnection()

   void packageAndSendRadiogram(){

      // ================================================================
      // package data into datagram and send
      // ----------------------------------------------------------------
      dg.reset();
      try {
         dg.writeLong(writeDataLong);
         dg.writeByte(writeDataByte);
         dg.writeUTF(location);
         dg.writeInt(writeDataInt);
         dg.writeDouble(writeDataDouble);
         dg.writeBoolean(writeDataBool);
         rCon.send(dg);
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Data packet sent.");
         // =============================================================


      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end packageAndSendRadiogram()

   void sleepForSomeTime(long samplePeriod, long lastTimeToBeSent) {
      // ================================================================
      // let the mote rest to conserve battery
      // ----------------------------------------------------------------
      Utils.sleep(samplePeriod - (System.currentTimeMillis() -
                 lastTimeToBeSent));
      // ================================================================
      
   } // end of sleepForSomeTime()

   long getCurrentTime(){
      // ================================================================
      // take time of sample
      // ----------------------------------------------------------------
      return System.currentTimeMillis();
      // ================================================================

   } // end getCurrentTime()

   double reformatDataToBeSent(double valueIn){
      /* ****************************************************************
      * Purpose     :  Method changes format of data of type long.
      * Purpose     :  Limit data to 2 digits after decimal point
      * Input       :  Data to be changed
      * Outputs     :  Data limited to two decimals
      * *****************************************************************
      */

      // ================================================================
      // local variable
      // ----------------------------------------------------------------
      double valueOut = 0;
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Print input value: " + valueIn);
      // ================================================================

      // ================================================================
      // Move decimal point 2 digits to the right
      // ----------------------------------------------------------------
      valueIn = valueIn * 100;
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("After * operator: " + valueIn);
      // ================================================================

      // ================================================================
      // Round up or down
      // ----------------------------------------------------------------
      valueIn = MathUtils.round(valueIn);
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("After rounding: " + valueIn);
      // ================================================================

      // ================================================================
      // Move decimal point 2 digits to the left
      // ----------------------------------------------------------------
      valueOut = valueIn / 100;
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("After / operator: " + valueOut);
      // ================================================================

      System.out.println("Data to be returned: "
              + valueOut);
      return valueOut;

   } // end of reformatTempToBeSent()

   void sampleEventIndication (boolean turnIndicationOn, int i) {
       if(turnIndicationOn){
           sampleEvent[i].setRGB(0, 0, 255);
           sampleEvent[i].setOn();              // LED on
       } else {
           sampleEvent[i].setOff();             // LED off
       } // end of IF/ ELSE
       
   } // end of sampleEventIndication ()

} // end CollectorMote class
