/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class runs if the MANIFEST.MF file has light ...
 * Purpose     :  measurement set to TRUE.
 * Inputs      :  Location of mote,
                  port used by host,
                  data type,
                  sample time
 * Outputs     :  Environmental data
 * **********************************************************************
 */

package org.sunspotworld;


class LightThread implements Runnable {
   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private String location;
   private Thread t;
   private int hostPortNumber;
   private byte dataIdentifier;
   private long sampleTime;
   private long timeToBeSent;
   private int lightToBeSent;
   // ===================================================================

   LightThread(String threadname, int portNo, byte dataType,
           long timeBetweenSamples) {
      location = threadname;
      t = new Thread(this, location);
      hostPortNumber = portNo;
      dataIdentifier = dataType;
      sampleTime = timeBetweenSamples;
      System.out.println("New thread: " + t);
      t.start();
   }
   
   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setLocThread(String setLoc){
      location = setLoc;
   }

   void setThread(Thread setThread){
      t = setThread;
   }

   void setPortThread(int setPort){
      hostPortNumber = setPort;
   }

   void setDataIdType(byte setDataType){
      dataIdentifier = setDataType;
   }

   void setTimeBetweenSamples(long setSampleInterval){
      sampleTime = setSampleInterval;
   }

   void setTimeForSending(long setSendTime){
      timeToBeSent = setSendTime;
   }

   void setLightForSending(int setSendLight){
      lightToBeSent = setSendLight;
   }
   // ===================================================================
   

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   String getLocThread(){
      return location;
   }

   Thread getThread(){
      return t;
   }

   int getPortThread(){
      return hostPortNumber;
   }

   byte getDataIdType(){
      return dataIdentifier;
   }

   long getTimeBetweenSamples(){
      return sampleTime;
   }

   long getTimeForSending(){
      return timeToBeSent;
   }

   int getLightForSending(){
      return lightToBeSent;
   }
   // ===================================================================


   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // entry point for light thread
   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   
   public void run() {
      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start light thread: " + location);
      // ================================================================

      // ================================================================
      // Start a new light level collector mote
      // ----------------------------------------------------------------
      CollectorLightMote moteLight = new CollectorLightMote();
      // ================================================================

      // ================================================================
      // Intial set up for measurement of light level
      // ----------------------------------------------------------------
      moteLight.initLightMeasurement();
      // ================================================================

      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      moteLight.initBroadcastCom();
      moteLight.setPortNumber(hostPortNumber);
      moteLight.setLocation(location);
      moteLight.setWriteDataByte(dataIdentifier);
      // ================================================================

      // ================================================================
      // get and push IEEE Address to terminal
      // ----------------------------------------------------------------
      moteLight.getPushAddress();
      // ================================================================

      // ================================================================
      // Open up a broadcast connection
      // ----------------------------------------------------------------
      moteLight.tryToOpenBroadcastConnection();
      // ================================================================

      // ================================================================
      // Get the current light reading and send in datagram
      // ----------------------------------------------------------------
      while (true){
         // =============================================================
         // Get the current time. I.e. time when sample is taken
         // -------------------------------------------------------------
         timeToBeSent = moteLight.getCurrentTime();
         // =============================================================

         // =============================================================
         // Indicate sample event (start, i.e LED on)
         // -------------------------------------------------------------
         moteLight.sampleEventIndication(true, 2);
         // =============================================================

         // =============================================================
         // Get the current light reading to send in datagram
         // -------------------------------------------------------------
         lightToBeSent = moteLight.takeLightSample();
         // =============================================================

         // =============================================================
         // Indicate sample event (end, i.e LED off)
         // -------------------------------------------------------------
         moteLight.sampleEventIndication(false, 2);
         // =============================================================

         // =============================================================
         // Convert raw data to luminance (lux). Based on Sun SPOT ...
         // Theory of Operation [p. 21] Luminance = 2 times raw data
         // -------------------------------------------------------------
         lightToBeSent = moteLight.convertRawDataToLux();

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Has light changed? " +
                 moteLight.hasLightChanged());
         // =============================================================

         if(moteLight.hasLightChanged()){
            // ==========================================================
            // For trouble shooting only
            // ----------------------------------------------------------
            System.out.println("Try to send sample.");
            // ==========================================================

            // ==========================================================
            // Update time and light value to be sent
            // ----------------------------------------------------------
            moteLight.setWriteDataLong(timeToBeSent);
            moteLight.setWriteDataInt(lightToBeSent);
            // ==========================================================

            // ==========================================================
            // Send current sample
            // ----------------------------------------------------------
            moteLight.packageAndSendRadiogram();
            // ==========================================================

         } // end if block

         // =============================================================
         // move current light sample to previous sample variable
         // -------------------------------------------------------------
         moteLight.moveCurrentLightToPreviousLight();
         // =============================================================

         // =============================================================
         // Go to sleep to conserve battery
         // -------------------------------------------------------------
         moteLight.sleepForSomeTime(sampleTime, timeToBeSent);
         // =============================================================

      } // end of while loop
   } // end of run()
} // end of class
