/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class runs if the MANIFEST.MF file has temp ...
 * Purpose     :  measurement set to TRUE.
 * Inputs      :  Location of mote,
                  port used by host,
                  data type,
                  sample time
 * Outputs     :  Collected environmental data
 * **********************************************************************
 */

package org.sunspotworld;


class TempThread implements Runnable {
   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private String location;
   private Thread t;
   private int hostPortNumber;
   private byte dataIdentifier;
   private long sampleTime;
   private long timeToBeSent;
   private double tempToBeSent;
   // ===================================================================

   TempThread(String threadname, int portNo, byte dataType,
           long timeBetweenSamples) {
      location = threadname;
      t = new Thread(this, location);
      hostPortNumber = portNo;
      dataIdentifier = dataType;
      sampleTime = timeBetweenSamples;
      System.out.println("New thread: " + t);
      t.start();
   }

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setLocThread(String setLoc){
      location = setLoc;
   }

   void setThread(Thread setThread){
      t = setThread;
   }

   void setPortThread(int setPort){
      hostPortNumber = setPort;
   }

   void setDataIdType(byte setDataType){
      dataIdentifier = setDataType;
   }

   void setTimeBetweenSamples(long setSampleInterval){
      sampleTime = setSampleInterval;
   }

   void setTimeForSending(long setSendTime){
      timeToBeSent = setSendTime;
   }

   void setTempForSending(double setSendTemp){
      tempToBeSent = setSendTemp;
   }
   // ===================================================================


   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   String getLocThread(){
      return location;
   }

   Thread getThread(){
      return t;
   }

   int getPortThread(){
      return hostPortNumber;
   }

   byte getDataIdType(){
      return dataIdentifier;
   }

   long getTimeBetweenSamples(){
      return sampleTime;
   }

   long getTimeForSending(){
      return timeToBeSent;
   }

   double getTempForSending(){
      return tempToBeSent;
   }
   // ===================================================================


   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   // entry point for temp thread
   // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

   public void run() {

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start temp thread: " + location);
      // ================================================================

      // ================================================================
      // start a new temperature collector mote
      // ----------------------------------------------------------------
      CollectorTempMote moteTemp = new CollectorTempMote();
      // ================================================================

      // ================================================================
      // Intial set up for measurement of temperature
      // ----------------------------------------------------------------
      moteTemp.initTemperatureMeasurement();
      // ================================================================

      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      moteTemp.initBroadcastCom();
      moteTemp.setPortNumber(hostPortNumber);
      moteTemp.setLocation(location);
      moteTemp.setWriteDataByte(dataIdentifier);
      // ================================================================

      // ================================================================
      // get and push IEEE Address to terminal
      // ----------------------------------------------------------------
      moteTemp.getPushAddress();
      // ================================================================

      // ================================================================
      // Open up a broadcast connection
      // ----------------------------------------------------------------
      moteTemp.tryToOpenBroadcastConnection();
      // ================================================================

      // ================================================================
      // Get the current temperature reading and send in datagram
      // ----------------------------------------------------------------
      while (true) {
         // =============================================================
         // Get the sample time
         // -------------------------------------------------------------
         timeToBeSent = moteTemp.getCurrentTime();
         // =============================================================

         // =============================================================
         // Indicate sample event (start, i.e LED on)
         // -------------------------------------------------------------
         moteTemp.sampleEventIndication(true, 1);
         // =============================================================

         // =============================================================
         // Get the current temperature reading to send in datagram
         // -------------------------------------------------------------
         tempToBeSent = moteTemp.takeTemperatureSample();
         // =============================================================

         // =============================================================
         // Indicate sample event (end, i.e LED off)
         // -------------------------------------------------------------
         moteTemp.sampleEventIndication(false, 1);
         // =============================================================

         // =============================================================
         // Reformat tempToBeSent to ****.**
         // -------------------------------------------------------------
         tempToBeSent = moteTemp.reformatDataToBeSent(tempToBeSent);
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Has temp changed? " +
                 moteTemp.hasTemperatureChanged());
         // =============================================================

         if(moteTemp.hasTemperatureChanged()){
            // ==========================================================
            // For trouble shooting only
            // ----------------------------------------------------------
            System.out.println("Try to send sample.");
            // ==========================================================

            // ==========================================================
            // Update time and temperatur value to be sent
            // ----------------------------------------------------------
            moteTemp.setWriteDataLong(timeToBeSent);
            moteTemp.setWriteDataDouble(tempToBeSent);
            // ==========================================================

            // ==========================================================
            // Send current sample
            // ----------------------------------------------------------
            moteTemp.packageAndSendRadiogram();
            // ==========================================================
            
         } // end if block

         // =============================================================
         // move current temp sample to previous sample variable
         // -------------------------------------------------------------
         moteTemp.moveCurrentTempToPreviousTemp();
         // =============================================================

         // =============================================================
         // Go to sleep to conserve battery
         // -------------------------------------------------------------
         moteTemp.sleepForSomeTime(sampleTime, timeToBeSent);
         // =============================================================

      } // end of while loop
   } // end of run()
} // end of class
