/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class includes functionalities to send and ...
 * Purpose     :  receive data.
 * **********************************************************************
 */

package org.sunspotworld;

import com.sun.spot.io.j2me.radiogram.RadiogramConnection;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;
import javax.microedition.io.Connector;
import javax.microedition.io.Datagram;


class CommClassHost {

   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private RadiogramConnection rCon;
   private Datagram dg;
   private String location;
   private int portNumber;

   private long writeDataLong = 0;
   private byte writeDataByte = 0;
   private int writeDataInt = 0;
   private double writeDataDouble = 0;
   private boolean writeDataBool = false;

   private long readDataLong = 0;
   private byte readDataByte = 0;
   private String readDataUTF = null;
   private int readDataInt = 0;
   private double readDataDouble = 0;
   private boolean readDataBool = false;

   DateFormat fmt = DateFormat.getTimeInstance();
   long timeOfReading = 0;
   long nowOnHost = 0L;
   long delaySampleToTerminal = 0L;
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setLocation(String setLoc){location = setLoc;}

   void setPortNumber(int setPort){portNumber = setPort;}

   void setWriteDataLong(long setDataLong){writeDataLong = setDataLong;}

   void setWriteDataByte(byte setDataByte){writeDataByte = setDataByte;}

   void setWriteDataInt(int setDataInt){writeDataInt = setDataInt;}

   void setWriteDataDouble(double setDataDouble){
      writeDataDouble = setDataDouble;}

   void setWriteDataBool(boolean setDataBool){
      writeDataBool = setDataBool;}

   // ===================================================================

   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   String getLocation(){return location;}

   int getPortNumber(){return portNumber;}

   long getWriteDataLong(){return writeDataLong;}

   byte getWriteDataByte(){return writeDataByte;}

   int getWriteDataInt(){return writeDataInt;}

   double getWriteDataDouble(){return writeDataDouble;}

   long getReadDataLong(){return readDataLong;}

   byte getReadDataByte(){return readDataByte;}

   String getReadDataUTF(){return readDataUTF;}

   int getReadDataInt(){return readDataInt;}

   double getReadDataDouble(){return readDataDouble;}

   boolean getReadDataBool(){return readDataBool;}
   // ===================================================================

   // ===================================================================
   // All other methods
   // ===================================================================
   void initBroadcastCom (){
      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      rCon = null;
      dg = null;
      // ================================================================

   } // end initBroadcastCom

   void tryToOpenBroadcastConnection(boolean broadRequired) {
      try {
         if (broadRequired) {
            // ==========================================================
            // Open up a broadcast connection.
            // ----------------------------------------------------------
            rCon = (RadiogramConnection) Connector.open(
                    "radiogram://broadcast:" + portNumber);
         // ==========================================================
         } else {
            // ==========================================================
            // Open up a unicast connection.
            // ----------------------------------------------------------
            rCon = (RadiogramConnection) Connector.open(
                    "radiogram://:" + portNumber);
         // ==========================================================
         }
         dg = rCon.newDatagram(rCon.getMaximumLength());
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Broadcast is open.");
      // =============================================================

      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end tryToOpenBroadcastConnection()

   void packageAndSendRadiogram(){

      // ================================================================
      // package data into datagram and send
      // ----------------------------------------------------------------
      dg.reset();
      try {
         dg.writeLong(writeDataLong);
         dg.writeByte(writeDataByte);
         dg.writeUTF(location);
         dg.writeInt(writeDataInt);
         dg.writeDouble(writeDataDouble);
         dg.writeBoolean(writeDataBool);
         rCon.send(dg);
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Data packet sent.");
         // =============================================================


      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end packageAndSendRadiogram()

   void listenFor(){

      // ================================================================
      // listen for new datagram data
      // ----------------------------------------------------------------
      try {
         rCon.receive(dg);
         readDataLong = dg.readLong();
         readDataByte = dg.readByte();
         readDataUTF = dg.readUTF();
         readDataInt = dg.readInt();
         readDataDouble = dg.readDouble();
         readDataBool = dg.readBoolean();
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Data packet received. " + readDataByte);
         // =============================================================

      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end listenFor()

   void pushTimeAndLocationToTerminal(boolean timeDelay){
      nowOnHost = System.currentTimeMillis();
      System.out.print("Time on Host: " +
              fmt.format(new Date(nowOnHost)) + " ");
     
      if(timeDelay){
         delaySampleToTerminal = nowOnHost - readDataLong;
         System.out.print(delaySampleToTerminal + " ms ");
         System.out.print(fmt.format(new Date(readDataLong)));
      }
      System.out.print("Location: " + readDataUTF + "\t");
   } // end of pushTimeAndLocationToTerminal()

   void identifyMessageAndPushToTerminal(int messageType){
      // ================================================================
      // Identify the data in the packets
      // ----------------------------------------------------------------
      switch (messageType) {
         case 1:
            System.out.println("Temperature (Celsius): " +
                    readDataDouble);
            break;
         case 2:
            System.out.println("Light (lux): " + readDataInt);
            break;
         case 3:
            System.out.println("Battery Capacity (%): " +
                    readDataInt);
            break;
         case 4:
            if (readDataInt == 1) {
               System.out.println("\tAlarm activated");
            } else if (readDataInt == 2) {
               System.out.println("\tNo alarm");
            } else {
               System.out.println("\tProblem, check SPOT");
            }
            break;
         case 5:
            System.out.println("Carbon Monoxide (Volt): " +
                    readDataDouble);
            break;
         case 111:
         case 112:
         case 113:
         case 114:
         case 115:
         case 116:
         case 117:
         case 118:
         case 119:
            if (readDataBool == true) {
               System.out.println(" Status --> OK");
            } else {
               System.out.println(" Status --> not OK");
            }
      } // end switch
      // ================================================================
      
   } // end of identifyMessageAndPushToTerminal()
  
} // end CollectorMote class
