/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class runs if MANIFEST.MF file has CarbonMon ...
 * Purpose     :  interface as a requirement.
 * Inputs      :  External System Type,
                  port used by host,
                  interface type as a byte
 * **********************************************************************
 */

package org.sunspotworld;

class CarbMonInterfaceThread implements Runnable {

   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private String systemInterface;
   private Thread t;
   private int hostPortNumber;
   private byte interfaceIdAsByte;
   private byte dataIdAsByte = 5;               // identifies CarMon data
   private boolean listenForNewDatagram = true;
   // ===================================================================

   CarbMonInterfaceThread(String threadname, int portNo, byte moteType) {
      systemInterface = threadname;
      t = new Thread(this, systemInterface);
      hostPortNumber = portNo;
      interfaceIdAsByte = moteType;
      System.out.println("New thread: " + t);
      t.start();
   } // end constructor

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setThread(Thread setThread) {
      t = setThread;
   }

   void setListenForNewDatagram(boolean lisForDgs) {
      listenForNewDatagram = lisForDgs;
   }
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   Thread getThread() {
      return t;
   }

   boolean getListenForNewDatagram() {
      return listenForNewDatagram;
   }
   // ===================================================================

   public void run() {
      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start interface thread: " +
              systemInterface + " " + interfaceIdAsByte);
      // ================================================================

      // ================================================================
      // start a new carbon mono interface mote
      // ----------------------------------------------------------------
      CarbMonInterfaceMote moteCarbMon = new CarbMonInterfaceMote();
      // ================================================================

      // ================================================================
      // get and push IEEE address
      // ----------------------------------------------------------------
      moteCarbMon.getPushAddress();
      // ================================================================

      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      moteCarbMon.initInterfaceMoteCommunication();
      moteCarbMon.setPortNumber(hostPortNumber);
      // ================================================================

      // ================================================================
      // Open up a broadcast connection
      // ----------------------------------------------------------------
      moteCarbMon.tryToOpenBroadcastConnection();
      // ================================================================

      // ================================================================
      // Intial set up to interface with external system
      // ----------------------------------------------------------------
      moteCarbMon.initExternalCommunication();
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start listening for datagrams on port: " +
              hostPortNumber);
      // ================================================================

      while (listenForNewDatagram) {
         // =============================================================
         // listen for new data packet
         // -------------------------------------------------------------
         moteCarbMon.listenFor();
         // =============================================================

         // =============================================================
         // Verify that packet has carbon mono data
         // -------------------------------------------------------------
         if (moteCarbMon.getReadDataByte() == dataIdAsByte) {
            // ==========================================================
            // For trouble shooting only
            // ----------------------------------------------------------
            System.out.println("Carbon mono data received.");
            // ==========================================================

            // ==========================================================
            // Prepare data to send to external system via serial comm.
            // ----------------------------------------------------------
            moteCarbMon.packageCarbonMonoDataToSendToCarbMonSystem(
                    moteCarbMon.getReadDataUTF(),
                    moteCarbMon.getReadDataDouble());

            // ==========================================================

            // ==========================================================
            // Push new data to external system
            // ----------------------------------------------------------
            moteCarbMon.sendDataToExternalSystem();
            // ==========================================================

         } // end of if packet has carbon mono data
         // =============================================================

      } // end of while loop

   } // end run()

} // end class
