/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Description :  Main class for the interface SPOTs.
 * Inputs      :  MANIFEST.MF file properties
 * Outputs     :  None
 * Purpose     :  Reads properties from MANIFEST.MF file & configures ...
 * Purpose     :  collector SPOT based on the MANIFEST.MF file.
 * Purpose     :  After the appropriate functions (threads) have been ...
 * Purpose     :  started the class waits for all threads to finish.
 * Purpose     :  When all treads are finished, class finishes.
 * **********************************************************************
 */

package org.sunspotworld;


import com.sun.spot.util.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;

public class InterfaceMainSPOT extends MIDlet {
   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private static final int PORT_LISTEN_HOST = 66;
   private static final int PORT_LISTEN_SPOT = 67;
   private static final int PORT_SEND_HOST = 68;

   private String MANI_INTERFACE = null;
   private String MANI_DOOR_OPEN_TIME = null;
   private byte interfaceByteId = 0;
   private byte expandByte = 100;
   private byte statusByteId = 0;
   private long doorOpenTime = 0;
   private String interfaceStringId = null;
   // ===================================================================

   protected void startApp() throws MIDletStateChangeException {
      // ================================================================
      // monitor the USB (if connected) and recognize commands from host
      // ----------------------------------------------------------------
      new BootloaderListener().start();
      // ================================================================

      // ================================================================
      // Get configuration data from Manifest file. After data has ...
      // ... been read from manifest it will be used to configure the ...
      // ... SPOT.
      // ----------------------------------------------------------------
      try {
         MANI_INTERFACE = this.getAppProperty("Interface");
         MANI_DOOR_OPEN_TIME = this.getAppProperty("DoorOpenTime");
         
         // -------------------------------------------------------------
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Configuration data OK");
         // -------------------------------------------------------------

      } catch (NullPointerException e) {
         System.out.println("Problem reading MANIFEST.MF " + e);
      } // end try / catch
      // ================================================================

      // ================================================================
      // convert Strings to appropriate data type
      // ----------------------------------------------------------------
      try {
         interfaceByteId = Byte.parseByte(MANI_INTERFACE);
         doorOpenTime = Long.parseLong(MANI_DOOR_OPEN_TIME);
      } catch (NumberFormatException numberFormatException) {
         System.out.println("Problem converting data: "
                 + numberFormatException
         );
      } // end try / catch
      // ================================================================

      // ================================================================
      // Find external system status id based on MANIFEST.MF file
      // ----------------------------------------------------------------
      statusByteId = (byte) (interfaceByteId + expandByte);
      // ================================================================
      
      // ================================================================
      // start new thread on a SunSPOT depending on MANIFEST.MF file
      // ----------------------------------------------------------------
      switch (interfaceByteId){
         case 11:
            interfaceStringId = "Climate Control";
            ClimateInterfaceThread climateInterMote =
                    new ClimateInterfaceThread(
                    interfaceStringId, PORT_LISTEN_SPOT,
                    interfaceByteId);

            // ==========================================================
            // Start thread to send status of external system to host
            // ----------------------------------------------------------
            SendStatus tempSysStatus = new SendStatus("statusClimate",
                    PORT_SEND_HOST, PORT_LISTEN_HOST, statusByteId);
            // ==========================================================

            try {
               // =======================================================
               // wait for thread to finish
               // -------------------------------------------------------
               climateInterMote.getThread().join();
               tempSysStatus.getThread().join();
               // =======================================================
            } catch (InterruptedException ex) {
               ex.printStackTrace();
            } // end of try / catch
            
            break;

         case 12:
            interfaceStringId = "Lighting Control";
            LightingInterfaceThread lightingInterMote =
                    new LightingInterfaceThread(
                    interfaceStringId, PORT_LISTEN_SPOT,
                    interfaceByteId);

            // ==========================================================
            // Start thread to send status of external system to host
            // ----------------------------------------------------------
            SendStatus lightSysStatus = new SendStatus("statusLight",
                    PORT_SEND_HOST, PORT_LISTEN_HOST, statusByteId);
            // ==========================================================

            try {
               // =======================================================
               // wait for thread to finish
               // -------------------------------------------------------
               lightingInterMote.getThread().join();
               lightSysStatus.getThread().join();
               // =======================================================
            } catch (InterruptedException ex) {
               ex.printStackTrace();
            } // end of try / catch

            break;

         case 14:
            interfaceStringId = "Alarm System";
            AlarmInterfaceThread alarmInterMote =
                    new AlarmInterfaceThread(
                    interfaceStringId, PORT_LISTEN_SPOT,
                    interfaceByteId);

            // ==========================================================
            // Start thread to send status of external system to host
            // ----------------------------------------------------------
            SendStatus alarmSysStatus = new SendStatus("statusAlarm",
                    PORT_SEND_HOST, PORT_LISTEN_HOST, statusByteId);
            // ==========================================================

            try {
               // =======================================================
               // wait for thread to finish
               // -------------------------------------------------------
               alarmInterMote.getThread().join();
               alarmSysStatus.getThread().join();
               // =======================================================
            } catch (InterruptedException ex) {
               ex.printStackTrace();
            } // end of try / catch

            break;

         case 15:
            interfaceStringId = "Carbon Mono System";
            CarbMonInterfaceThread ventInterMote =
                    new CarbMonInterfaceThread(
                    interfaceStringId, PORT_LISTEN_SPOT,
                    interfaceByteId);

            // ==========================================================
            // Start thread to send status of external system to host
            // ----------------------------------------------------------
            SendStatus carbMonSysStatus = new SendStatus("statusCarbMon",
                    PORT_SEND_HOST, PORT_LISTEN_HOST, statusByteId);
            // ==========================================================

            try {
               // =======================================================
               // wait for thread to finish
               // -------------------------------------------------------
               ventInterMote.getThread().join();
               carbMonSysStatus.getThread().join();
               // =======================================================
            } catch (InterruptedException ex) {
               ex.printStackTrace();
            } // end of try / catch

            break;

         case 16:
            interfaceStringId = "Door Interface";
            DoorInterfaceThread doorInterMote =
                    new DoorInterfaceThread(
                    interfaceStringId, PORT_LISTEN_SPOT, interfaceByteId,
                    doorOpenTime);

            // ==========================================================
            // Start thread to send status of external system to host
            // ----------------------------------------------------------
            SendStatus doorSysStatus = new SendStatus("statusDoor",
                    PORT_SEND_HOST, PORT_LISTEN_HOST, statusByteId);
            // ==========================================================

            try {
               // =======================================================
               // wait for thread to finish
               // -------------------------------------------------------
               doorInterMote.getThread().join();
               doorSysStatus.getThread().join();
               // =======================================================
            } catch (InterruptedException ex) {
               ex.printStackTrace();
            } // end of try / catch

            break;

      } // end of switch (interfaceByteId)
      // ================================================================

   } // end of startApp()

   protected void pauseApp() {
      // This is not currently called by the Squawk VM

   } // end of pauseApp()

  protected void destroyApp(boolean arg0) throws
          MIDletStateChangeException {
      
   } // end of destroyApp()
} // end of class
