/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class includes all the common functions of a ...
 * Purpose     :  interface mote. Usually this class will be extended ...
 * Purpose     :  to add the specific capability for the interface ...
 * Purpose     :  to external systems.
 * **********************************************************************
 */

package org.sunspotworld;

import com.sun.spot.io.j2me.radiogram.RadiogramConnection;
import com.sun.spot.sensorboard.EDemoBoard;
import com.sun.spot.sensorboard.peripheral.ITriColorLED;
import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.Datagram;
import com.sun.spot.util.Utils;

class InterfaceMote {

   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private String systemInterface;
   private int portNumber;
   private String ourAddress;
   private RadiogramConnection rCon;
   private Datagram dg;

   private long readDataLong = 0;
   private byte readDataByte = 0;
   private String readDataUTF = null;
   private int readDataInt = 0;
   private double readDataDouble = 0;
   private boolean readDataBool = false;

   EDemoBoard serialComm = EDemoBoard.getInstance();
   private int baud = 19200;
   private int databits = 8;
   private int parity = 1;          // 0 = none, 1 = odd, 2 = even
   private int stopbits = 1;
   private String externalDataString = null;

   private ITriColorLED[] sendCommIndication = EDemoBoard.
           getInstance().getLEDs();

   // ===================================================================

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setSystemInterface(String setLoc){
      systemInterface = setLoc;
   }

   void setPortNumber(int setPort){
      portNumber = setPort;
   }

   void setBaud (int baudValue){
      baud = baudValue;
   }

   void setDatabits(int databitsValue){
      databits = databitsValue;
   }

   void setParity (int parityValue) {
      parity = parityValue;
   }

   void setStopbits (int stopbitsValue) {
      stopbits = stopbitsValue;
   }

   void setExternalDataString (String dataOut) {
      externalDataString = dataOut;
   }

   // ===================================================================

   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   String getSystemInterface(){
      return systemInterface;
   }

   String getOurAddress(){
      return ourAddress;
   }

   int getPortNumber(){
      return portNumber;
   }

   long getReadDataLong(){
      return readDataLong;
   }

   byte getReadDataByte(){
      return readDataByte;
   }

   String getReadDataUTF(){
      return readDataUTF;
   }

   int getReadDataInt(){
      return readDataInt;
   }

   double getReadDataDouble(){
      return readDataDouble;
   }

   boolean getReadDataBoolean(){
      return readDataBool;
   }

   int getBaud (){
      return baud;
   }

   int getDatabits(){
      return databits;
   }

   int getParity () {
      return parity;
   }

   int getStopbits () {
      return stopbits;
   }

   String getExternalDataString () {
      return externalDataString;
   }

   // ===================================================================


   // ===================================================================
   // All other methods
   // ===================================================================
   void initInterfaceMoteCommunication (){
      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      rCon = null;
      dg = null;
      // ================================================================

   } // end initBroadcastCom

   void getPushAddress() {
      // ================================================================
      // get and push IEEE Address to terminal. Troubleshoot only
      // ----------------------------------------------------------------
      ourAddress = System.getProperty("IEEE_ADDRESS");
      System.out.println("Starting app. on " + ourAddress + " ...");
      // ================================================================

   } // end getPushAddress()

   void tryToOpenBroadcastConnection(){
      try {
         // =============================================================
         // Open up a radiogram connection to listen for data from ...
         // collector motes.
         // -------------------------------------------------------------
         rCon = (RadiogramConnection) Connector.open(
              "radiogram://:" + portNumber);
         dg = rCon.newDatagram(rCon.getMaximumLength());
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Radiogramm connection is open.");
         // =============================================================
         
      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
      
   } // end tryToOpenBroadcastConnection()

   void listenFor(){

      // ================================================================
      // listen for new datagram data
      // ----------------------------------------------------------------
      try {
         rCon.receive(dg);
         readDataLong = dg.readLong();
         readDataByte = dg.readByte();
         readDataUTF = dg.readUTF();
         readDataInt = dg.readInt();
         readDataDouble = dg.readDouble();
         readDataBool = dg.readBoolean();
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Data packet received. " + readDataByte);
         // =============================================================


      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end listenFor()

   void sleepForSomeTime(long samplePeriod, long lastTimeToBeSent) {
      // ================================================================
      // let the mote rest to conserve battery, probably never called ...
      // in interface mote.
      // ----------------------------------------------------------------
      Utils.sleep(samplePeriod - (System.currentTimeMillis() -
                 lastTimeToBeSent));
      // ================================================================
      
   } // end of sleepForSomeTime()

   long getCurrentTime(){
      // ================================================================
      // take time of sample
      // ----------------------------------------------------------------
      return System.currentTimeMillis();
      // ================================================================

   } // end getCurrentTime()

   void initExternalCommunication(){
      // ================================================================
      // For troubleshooting only
      // ----------------------------------------------------------------
      System.out.println("Baud\t: " + baud);
      System.out.println("Databits\t: " + databits);
      System.out.println("Parity\t: " + parity);
      System.out.println("Stop bits\t: " + stopbits);

      // ================================================================
      // Init serial communication to external system
      // ----------------------------------------------------------------
      serialComm.initUART(baud, databits, parity, stopbits);
      // ================================================================

   } // end of initExternalCommunication()

  void sendDataToExternalSystem(){
      // ================================================================
      // Send data via serial communication to external system
      // ----------------------------------------------------------------
      for (int i = 0; i<8; i++){
         sendCommIndication[i].setRGB(0, 0, 255);
         sendCommIndication[i].setOn();
      } // set each LED to blue and turn on
      
      serialComm.writeUART(externalDataString);

      for (int i = 7; i >= 0; i--){
         sendCommIndication[i].setOff();
      } // turn LEDs off

      
      System.out.println("Data sent via serial comm to external system");
      // ================================================================
   } // end of sendDataToExternalSystem()

} // end CollectorMote class
