/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class runs if the MANIFEST.MF file has light ...
 * Purpose     :  interface as a requirement.
 * Inputs      :  External System Type,
                  port used by host,
                  interface type as a byte
 * **********************************************************************
 */

package org.sunspotworld;

class LightingInterfaceThread implements Runnable {
   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private String systemInterface;
   private Thread t;
   private int hostPortNumber;
   private byte interfaceIdAsByte;
   private byte dataIdAsByte = 2;               // identifies light data
   private boolean listenForNewDatagram = true;
   // ===================================================================


   LightingInterfaceThread(String threadname, int portNo, byte moteType){
      systemInterface = threadname;
      t = new Thread(this, systemInterface);
      hostPortNumber = portNo;
      interfaceIdAsByte = moteType;
      System.out.println("New thread: " + t);
      t.start();
   } // end constructor

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setThread(Thread setThread) {
      t = setThread;
   }

   void setListenForNewDatagram(boolean lisForDgs) {
      listenForNewDatagram = lisForDgs;
   }
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   Thread getThread() {
      return t;
   }

   boolean getListenForNewDatagram() {
      return listenForNewDatagram;
   }
   // ===================================================================

   public void run() {
      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start interface thread: " +
              systemInterface + " " + interfaceIdAsByte);
      // ================================================================

      // ================================================================
      // start a new light interface mote
      // ----------------------------------------------------------------
      LightInterfaceMote moteLight = new LightInterfaceMote();
      // ================================================================

      // ================================================================
      // get and push IEEE address to terminal
      // ----------------------------------------------------------------
      moteLight.getPushAddress();
      // ================================================================

      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      moteLight.initInterfaceMoteCommunication();
      moteLight.setPortNumber(hostPortNumber);
      // ================================================================

      // ================================================================
      // Open up a broadcast connection
      // ----------------------------------------------------------------
      moteLight.tryToOpenBroadcastConnection();
      // ================================================================
      
      // ================================================================
      // Intial set up to communicate with external system
      // ----------------------------------------------------------------
      moteLight.initExternalCommunication();
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start listening for datagrams on port: " +
              hostPortNumber);
      // ================================================================

      while (listenForNewDatagram) {
         // =============================================================
         // listen for new data packet
         // -------------------------------------------------------------
         moteLight.listenFor();
         // =============================================================

         // =============================================================
         // Verify that packet has light data
         // -------------------------------------------------------------
         if (moteLight.getReadDataByte() == dataIdAsByte) {
            // ==========================================================
            // For trouble shooting only
            // ----------------------------------------------------------
            System.out.println("Light data received.");
            // ==========================================================

            // ==========================================================
            // Prepare data to send to external system via serial comm.
            // ----------------------------------------------------------
            moteLight.packageLightDataToSendToLightControl(
                    moteLight.getReadDataUTF(),
                    moteLight.getReadDataInt());
            // ==========================================================

            // ==========================================================
            // Push new data to external system
            // ----------------------------------------------------------
            moteLight.sendDataToExternalSystem();
            // ==========================================================

         } // end of if packet has light data
         // =============================================================

      } // end of while loop

   } // end run()

} // end class
