/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This class includes all the special functionalities ...
 * Purpose     :  to send the status of an external system to the host.
 * **********************************************************************
 */
package org.sunspotworld;

import com.sun.spot.io.j2me.radiogram.RadiogramConnection;
import com.sun.spot.sensorboard.EDemoBoard;
import com.sun.spot.sensorboard.EDemoController;
import com.sun.spot.sensorboard.io.InputPin;
import com.sun.spot.sensorboard.io.PinDescriptor;
import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.Datagram;

class RadiogramCommClass {
   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private RadiogramConnection rCon;
   private Datagram dg;

   private PinDescriptor statusExternalSystem;
   private EDemoBoard status;
   private EDemoController statusController;
   private InputPin pin;

   private boolean extSysStatus = false;
   private String location;
   private String hostAddress;
   private int portNumber;

   private long writeDataLong = 0;
   private byte writeDataByte = 0;
   private int writeDataInt = 0;
   private double writeDataDouble = 0;
   private boolean writeDataBool = false;
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   InputPin getPinStatus() {
      return pin;
   }

   void setLocation(String setLoc) {
      location = setLoc;
   }

   void setPortNumber(int setPort) {
      portNumber = setPort;
   }

   void setHostAddress(String hostAddr) {
      hostAddress = hostAddr;
   }

   void setWriteDataLong(long setDataLong) {
      writeDataLong = setDataLong;
   }

   void setWriteDataByte(byte setDataByte) {
      writeDataByte = setDataByte;
   }

   void setWriteDataInt(int setDataInt) {
      writeDataInt = setDataInt;
   }

   void setWriteDataDouble(double setDataDouble) {
      writeDataDouble = setDataDouble;
   }

   void setWriteBoolean(boolean setDataBoolean) {
      writeDataBool = setDataBoolean;
   }
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   String getLocation() {
      return location;
   }

   int getPortNumber() {
      return portNumber;
   }

   String getHostAddress() {
      return hostAddress;
   }

   long getWriteDataLong() {
      return writeDataLong;
   }

   byte getWriteDataByte() {
      return writeDataByte;
   }

   int getWriteDataInt() {
      return writeDataInt;
   }

   double getWriteDataDouble() {
      return writeDataDouble;
   }

   boolean getWriteDataBoolean() {
      return writeDataBool;
   }

   // ===================================================================

   // ===================================================================
   // All other methods
   // ===================================================================
   void initRadiogramCom() {
      // ================================================================
      // Intial set up for communication
      // ----------------------------------------------------------------
      rCon = null;
      dg = null;
      // ================================================================

   } // end initRadiogramCom()

   void tryToOpenRadiogramConnection() {
      try {
         // =============================================================
         // Open up a radiogram connection to the host
         // -------------------------------------------------------------
         rCon = (RadiogramConnection) Connector.open(
                 "radiogram://" + hostAddress + ":" + portNumber);
         ((RadiogramConnection) rCon).setMaxBroadcastHops(4);
         dg = rCon.newDatagram(rCon.getMaximumLength());
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Broadcast is open.");
         // =============================================================

      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end tryToOpenaRadiogramConnection()

   void initExternalSystemStatus() {
      statusExternalSystem = EDemoController.D2;
      status = EDemoBoard.getInstance();
      statusController = status.getEDemoController();
      pin = new InputPin(statusExternalSystem, statusController);
      statusController.setPinDirection(statusExternalSystem, false);
      statusController.enablePinChangeInterrupts(pin);

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("External system status initialised.");
      // ================================================================

   } // end of initExternalSystemStatus()

   boolean errorExternalSystem() {
      // ================================================================
      // Check if input is low on D2. Low = external System OK
      // ----------------------------------------------------------------
      extSysStatus = pin.isLow();
      // ================================================================

      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("External System Status: " + extSysStatus);
      // ================================================================

      return extSysStatus;
      // ================================================================

   } // end of errorExternalSystem()

   void packageAndSendRadiogram() {

      // ================================================================
      // package data into datagram and send
      // ----------------------------------------------------------------
      dg.reset();
      try {
         dg.writeLong(writeDataLong);
         dg.writeByte(writeDataByte);
         dg.writeUTF(location);
         dg.writeInt(writeDataInt);
         dg.writeDouble(writeDataDouble);
         dg.writeBoolean(writeDataBool);
         rCon.send(dg);
         // =============================================================

         // =============================================================
         // For trouble shooting only
         // -------------------------------------------------------------
         System.out.println("Data packet sent.");
         // =============================================================


      } catch (IOException ex) {
         ex.printStackTrace();
      } // end try / catch
   } // end packageAndSendRadiogram()
} // end of RadiogramCommClass()
