/* **********************************************************************
 * Author      :  Severin Willisch
 * email       :  d9840087@mail.connect.usq.edu.au
 * Student No. :  0039840087
 * Purpose     :  Project for Semester 1 & 2, 2009
 * Purpose     :  09-036 Programming Sun SPOTs in JAVA
 * **********************************************************************
 */

/* **********************************************************************
 * Purpose     :  This thread sends the status of the external system ...
 * Purpose     :  to the host.
 * **********************************************************************
 */

package org.sunspotworld;

import com.sun.spot.util.IEEEAddress;

class SendStatus implements Runnable{

   // ===================================================================
   // Instance variables
   // -------------------------------------------------------------------
   private String systemInterface;
   private Thread t;
   private int portNumberSend;
   private int portNumberListen;
   private byte statusIdentification;
   private String hostAddress;

   private boolean extSystemOK = false;
   private boolean hostAddressNotKnown = true;
   // ===================================================================

   SendStatus(String threadname, int portSend, int portList,
           byte byteId){
      systemInterface = threadname;
      t = new Thread(this, systemInterface);
      portNumberSend = portSend;
      portNumberListen = portList;
      statusIdentification = byteId;
      System.out.println("New thread: " + t);
      t.start();
   } // end constructor


   // ===================================================================
   // methods to access instance variables, setter methods
   // ===================================================================
   void setThread(Thread setThread) {
      t = setThread;
   }
   // ===================================================================

   // ===================================================================
   // methods to access instance variables, getter methods
   // ===================================================================
   Thread getThread() {
      return t;
   }
   // ===================================================================

   public void run() {
      // ================================================================
      // For trouble shooting only
      // ----------------------------------------------------------------
      System.out.println("Start external system status thread: " +
              systemInterface);
      System.out.println("Listening on port: " + portNumberListen);
      // ================================================================

      // ================================================================
      // Create object to send status to host
      // ----------------------------------------------------------------
      InterfaceMote listenForHost = new InterfaceMote();
      // ================================================================

      // ================================================================
      // Initial setup for communication (listening)
      // ----------------------------------------------------------------
      listenForHost.initInterfaceMoteCommunication();
      listenForHost.setPortNumber(portNumberListen);
      // ================================================================

      // ================================================================
      // Open up a broadcast connection to listen for host
      // ----------------------------------------------------------------
      listenForHost.tryToOpenBroadcastConnection();
      // ================================================================

      // ================================================================
      // Wait for host address
      // ----------------------------------------------------------------
      while (hostAddressNotKnown){
         listenForHost.listenFor();
         if(listenForHost.getReadDataByte() == 99){
            hostAddress = IEEEAddress.toDottedHex
                    (listenForHost.getReadDataLong());
            
            // ==========================================================
            // For trouble shooting only
            // ----------------------------------------------------------
            System.out.println("Host address received: " + hostAddress);
            // ==========================================================

            // ==========================================================
            // Set flag to false (i.e. address is now known)
            // ----------------------------------------------------------
            hostAddressNotKnown = false;
            // ==========================================================
            
            try {
               Thread.sleep(2000);
            } catch (InterruptedException ex) {
               ex.printStackTrace();
            }
         } // end of IF

      } // end of WHILE
      // ================================================================

      // ================================================================
      // Create object to send status to host
      // ----------------------------------------------------------------
      RadiogramCommClass sendExternalSystemStatus =
              new RadiogramCommClass();
      // ================================================================

      // ================================================================
      // Initial setup for communication (sending)
      // ----------------------------------------------------------------
      sendExternalSystemStatus.initRadiogramCom();
      sendExternalSystemStatus.setPortNumber(portNumberSend);
      sendExternalSystemStatus.setLocation(systemInterface);
      sendExternalSystemStatus.setHostAddress(hostAddress);
      sendExternalSystemStatus.setWriteDataByte(statusIdentification);
      // ================================================================

      // ================================================================
      // Open up a radiogram connection to host
      // ----------------------------------------------------------------
      sendExternalSystemStatus.tryToOpenRadiogramConnection();
      // ================================================================

      // ================================================================
      // Intial set up to verify status of external system
      // ----------------------------------------------------------------
      sendExternalSystemStatus.initExternalSystemStatus();
      // ================================================================

      while (true){

         // =============================================================
         // check status of external system
         // -------------------------------------------------------------
         extSystemOK = sendExternalSystemStatus.errorExternalSystem();
         sendExternalSystemStatus.setWriteBoolean(extSystemOK);
         // =============================================================

         // =============================================================
         // send status to host
         // -------------------------------------------------------------
         sendExternalSystemStatus.packageAndSendRadiogram();
         
         try {

            Thread.sleep(30000);
         } catch (InterruptedException ex) {
            ex.printStackTrace();
         }

      } // end of WHILE

   } // end of SendStatus

} // end of class
